@@ -16,18 +16,12 @@ class PartpyError(Exception):
         self.partpyobj = obj
 
     def __repr__(self):
-        output = [str(line) for line in \
-            self.partpyobj.get_surrounding_lines(1, 0)]
+        output = ['\n']
+        output.extend([str(line) for line in \
+            self.partpyobj.get_surrounding_lines(1, 0)])
 
-        padding = 1
-        if self.partpyobj.row < 1000:
-            padding = 2
-        if self.partpyobj.row < 100:
-            padding = 3
-        if self.partpyobj.row < 10:
-            padding = 4
-
-        output.append(' ' * (self.partpyobj.col + padding + 1) + '^' + '\n')
+        output.append('\n' + \
+            (' ' * (self.partpyobj.col + 5)) + '^' + '\n')
         if self.partpymsg:
             output.append(self.partpymsg)
 
