@@ -5,6 +5,8 @@ for matching strings and patterns.
 __author__ = 'Taylor "Nekroze" Lawson'
 __email__ = 'nekroze@eturnilnetwork.com'
 
+from .partpyerror import PartpyError
+
 
 class SourceString(object):
     """Stores the parse string and its length followed by current position
@@ -16,14 +18,17 @@ class SourceString(object):
     the source string.
     """
     def __init__(self, string = None):
+        """Accepts a string or None by default. If a string is given then
+        self.set_string(string) is run automatically. If you wish to load a file
+        then create a SourceString object with no arguments and then use
+        load_file or overload this function when inheriting from SourceString
+        """
         self.string = ''
         self.length = 0
-        self.pos = 0
-        self.col = 0
-        self.row = 0
-        self.eos = 0
         if string is not None:
             self.set_string(string)
+        else:
+            self.reset_position()
 
     def load_file(self, filename):
         """Read in file contents and set the current string."""
@@ -46,34 +51,67 @@ class SourceString(object):
         """Reset all current positions."""
         self.pos = 0
         self.col = 0
-        self.row = 0
+        self.row = 1
         self.eos = 0
 
     def has_space(self, length = 1):
         """Returns boolean if self.pos + length < working string length."""
-        return self.pos + length-1 < self.length
+        return self.pos + length - 1 < self.length
 
     def eat_length(self, length):
         """Move current position by length and set eos if not has_space()."""
+        pos = self.pos
+        if self.eos or pos + length > self.length:
+            return None
 
-        self.col += length
-        self.pos += length
+        col = self.col
+        row = self.row
+        for char in self.string[pos:pos + length]:
+            col += 1
+            pos += 1
+            if char == '\n':  # handle a newline char
+                col = 0
+                row += 1
+
+        self.pos = pos
+        self.col = col
+        self.row = row
 
         if not self.has_space():  # Set eos if there is no more space left.
             self.eos = 1
 
     def eat_string(self, string):
         """Move current position by length of string and count lines by \n."""
-        if string == '\n':  # Handle single newline.
-            self.col = -1
-            self.row += 1
-            self.eat_length(1)
-        elif '\n' in string:  # Handle string containing a newline.
-            for char in string:  # Recursively call eat to handle each char.
-                self.eat_string(char)
-        else:
-            length = len(string)
-            self.eat_length(length)  # Any other string just eat the length.
+        pos = self.pos
+        if self.eos or pos + len(string) > self.length:
+            return None
+
+        col = self.col
+        row = self.row
+        for char in string:
+            col += 1
+            pos += 1
+            if char == '\n':  # handle a newline char
+                col = 0
+                row += 1
+
+        self.pos = pos
+        self.col = col
+        self.row = row
+
+        if not self.has_space():  # Set eos if there is no more space left.
+            self.eos = 1
+
+    def eat_line(self):
+        """Move current position forward until the next line."""
+        if self.eos:
+            return None
+        eat_length = self.eat_length
+        get_char = self.get_char
+        has_space = self.has_space
+        while has_space() and get_char() != '\n':
+            eat_length(1)
+        eat_length(1)
 
     def get_char(self):
         """Return the current character in the working string."""
@@ -84,30 +122,27 @@ class SourceString(object):
 
     def get_length(self, length, trim = 0):
         """Return string at current position + length.
-        If trim == true then get as much as possible before eos"""
-        if not self.has_space():
+        If trim == true then get as much as possible before eos.
+        """
+        if trim and not self.has_space(length):
+            return self.string[self.pos:]
+        elif self.has_space(length):
+            return self.string[self.pos:self.pos + length]
+        else:
             return ''
 
-        pos = self.pos
-        distance = pos + length
-        if not trim and not self.has_space(length):
-            return ''
-        return self.string[pos:distance]
-
     def get_string(self):
         """Return non space chars from current position until a whitespace."""
         if not self.has_space():
             return ''
 
-        pos = self.pos
-        string = self.string
         # Get a char for each char in the current string from pos onward
         #  solong as the char is not whitespace.
         # The following is not yet supported with cython 0.18.0
         #from itertools import takewhile
         #chars = (y for y in takewhile(lambda x: not x.isspace(), string[pos:]))
         chars = []
-        for char in string[pos:]:
+        for char in self.string[self.pos:]:
             if char.isspace():
                 break
             else:
@@ -121,7 +156,10 @@ class SourceString(object):
 
     def rest_of_string(self, offset = 0):
         """A copy of the current position till the end of the string."""
-        return self.string[self.pos + offset:]
+        if self.has_space(offset):
+            return self.string[self.pos + offset:]
+        else:
+            return ''
 
     def get_line(self, lineno):
         """Return any line as a SourceLine and None if lineno doesnt exist."""
@@ -152,21 +190,24 @@ class SourceString(object):
             pos += 1
             if pos == end:
                 break
+        else:
+            output.append(string[pos])
+
         if not output:
             return None
 
-        return SourceLine(''.join(output) + '\n', self.row)
+        return SourceLine(''.join(output), self.row)
 
     def get_lines(self, first, last):
         """Return SourceLines for lines between and including first and last."""
-        line = 0
+        line = 1
         linestring = []
         linestrings = []
         for char in self.string:
             if line >= first and line <= last:
                 linestring.append(char)
                 if char == '\n':
-                    linestrings.append(''.join(linestring))
+                    linestrings.append((''.join(linestring), line))
                     linestring = []
             elif line > last:
                 break
@@ -174,16 +215,16 @@ class SourceString(object):
             if char == '\n':
                 line += 1
         if linestring:
-            linestrings.append(''.join(linestring))
+            linestrings.append((''.join(linestring), line))
         elif not linestrings:
             return None
 
-        return [SourceLine(line, first + num) for num, line in \
-            enumerate(linestrings)]
+        return [SourceLine(string, lineno) for string, lineno in linestrings]
 
     def get_surrounding_lines(self, past = 1, future = 1):
         """Return the current line and x,y previous and future lines.
-        Returns a list of SourceLine's"""
+        Returns a list of SourceLine's.
+        """
         string = self.string
         pos = self.pos - self.col
         end = self.length
@@ -218,11 +259,28 @@ class SourceString(object):
 
         return output
 
+    def get_all_lines(self):
+        """Return all lines of the SourceString as a list of SourceLine's."""
+        output = []
+        line = []
+        lineno = 1
+        for char in self.string:
+            line.append(char)
+            if char == '\n':
+                output.append(SourceLine(''.join(line), lineno))
+                line = []
+                lineno += 1
+        if line:
+            output.append(SourceLine(''.join(line), lineno))
+
+        return output
+
     def match_string(self, string, word = 0):
         """Returns 1 if string can be matches against SourceString's
         current position.
 
-        If word is >= 1 then it will only match string followed by whitepsace"""
+        If word is >= 1 then it will only match string followed by whitepsace.
+        """
         if word:
             return self.get_string() == string
         return self.get_length(len(string)) == string
@@ -230,15 +288,16 @@ class SourceString(object):
     def match_any_string(self, strings, word = 0):
         """Attempts to match each string in strings in order of length.
         Will return the string that matches or an empty string if no match.
-        Sorts strings list by string length, consider immutability.
+        Sorts strings list by string length internally.
 
-        if Word then only match if string is followed by a whitespace."""
-        current = ''
+        if Word then only match if string is followed by a whitespace.
+        """
         if word:
             current = self.get_string()
-            return current if self.get_string() in strings else ''
+            return current if current in strings else ''
 
         strings = sorted(strings, key = len)
+        current = ''
 
         currentlength = 0
         length = 0
@@ -252,17 +311,23 @@ class SourceString(object):
 
     def match_any_char(self, chars):
         """Match and return the current SourceString char if its in chars."""
+        if not self.has_space():
+            return ''
         current = self.string[self.pos]
         return current if current in chars else ''
 
-    def match_pattern(self, first, rest = None):
+    def match_pattern(self, first, rest = None, least = 1):
         """Match each char sequentially from current SourceString position
         until the pattern doesnt match and return all maches.
 
         First may be a list or tuple that will get unpacked to first, rest.
 
+        Integer argument least defines and minimum amount of chars that can
+        be matched.
+
         If rest is defined then first is used only to match the first arg
-        and the rest of the chars are matched against rest."""
+        and the rest of the chars are matched against rest.
+        """
         ftype = type(first)
         if rest is None and ftype in (tuple, list):
             first, rest = first
@@ -279,20 +344,28 @@ class SourceString(object):
                 output.append(char)
             else:
                 break
+
+        if len(output) < least:
+            return ''
+
         return ''.join(output)
 
-    def match_function(self, first, rest = None):
+    def match_function(self, first, rest = None, least = 1):
         """Match each char sequentially from current SourceString position
         until the pattern doesnt match and return all maches.
 
         First may be a list or tuple that will get unpacked to first, rest.
 
+        Integer argument least defines and minimum amount of chars that can
+        be matched.
+
         This version takes functions instead of string patterns.
         Each function must take one argument, a string, and return a
         value that can be evauluated as True or False.
 
         If rest is defined then first is used only to match the first arg
-        and the rest of the chars are matched against rest."""
+        and the rest of the chars are matched against rest.
+        """
         ftype = type(first)
         if rest is None and ftype in (tuple, list):
             first, rest = first
@@ -309,11 +382,16 @@ class SourceString(object):
                 output.append(char)
             else:
                 break
+
+        if len(output) < least:
+            return ''
+
         return ''.join(output)
 
     def count_indents(self, spacecount, tabs = 0):
         """Counts the number of indents that can be tabs or spacecount
-        number of spaces in a row from the current position."""
+        number of spaces in a row from the current position.
+        """
         spaces = 0
         indents = 0
         for char in self.string[self.pos - self.col:]:
@@ -352,9 +430,112 @@ class SourceString(object):
                 spaces = 0
         return (indents, charlen)
 
+    def count_indents_last_line(self, spacecount, tabs = 0, back = 5):
+        """Finds the last meaningful line and returns its indent level."""
+        lines = self.get_surrounding_lines(back, 0)
+
+        for line in reversed(lines):
+            if not line.string.isspace():
+                return line.count_indents(spacecount, tabs)
+        return 0
+
+    def count_indents_length_last_line(self, spacecount, tabs = 0, back = 5):
+        """Finds the last meaningful line and returns its indent level and
+        character length.
+        """
+        lines = self.get_surrounding_lines(back, 0)
+
+        for line in reversed(lines):
+            if not line.string.isspace():
+                return line.count_indents_length(spacecount, tabs)
+        return (0, 0)
+
+    def skip_whitespace(self, newlines = 0):
+        """Moves the position forwards to the next non newline space character.
+        If newlines >= 1 include newlines as spaces.
+        """
+        if newlines:
+            while not self.eos:
+                if self.get_char().isspace():
+                    self.eat_length(1)
+                else:
+                    break
+        else:
+            char = ''
+            while not self.eos:
+                char = self.get_char()
+                if char.isspace() and char != '\n':
+                    self.eat_length(1)
+                else:
+                    break
+
+    def retrieve_tokens(self, matcher, tokens, longest = 1, newlines = 1):
+        """Moves through SourceString and attempts to match and yield tokens
+        while skipping over whitespace and newlines unless newlines == 0.
+        Uses the longest match by default set longest to 0 to match shortest.
+        The matcher argument is either match_pattern or match_function.
+
+        Currently yield is not supported by cython so this function is
+        un-optimized but is still compiled. As such this could use more pure
+        python optimizations.
+        """
+        matches = {}
+        ordered = []
+        matched = ''
+        index = -1 if longest else 0
+
+        while not self.eos:
+            self.skip_whitespace(newlines)
+            for token, pattern in tokens.items():
+                matched = matcher(pattern)
+                if matched:
+                    matches[matched] = token
+
+            if len(matches):
+                ordered = sorted(matches.keys(), key = len)
+                match = ordered[index]
+                yield (matches[match], match)
+                self.eat_length(len(match))
+            else:
+                raise PartpyError(self, 'No token pattern for current char.')
+            matched = ''
+            matches = {}
+            match = ''
+            ordered = []
+
+
     def __repr__(self):
+        """Returns the entire base string. Called from the repr() builtin."""
         return self.string
 
+    def __getitem__(self, index):
+        """Returns the character at the given index.
+        Called by SourceString[index] where index is an integer.
+        """
+        return self.string[index]
+
+    def __delitem__(self, index):
+        del self.string[index]
+
+    def __setitem__(self, index, value):
+        self.string[index] = value
+
+    def __len__(self):
+        """Returns the length of base string. Called by len(SourceString)."""
+        return len(self.string)
+
+    def __contains__(self, string):
+        """Returns a boolean if the given string is within the base string.
+        Called by 'word' in SourceString.
+        """
+        return string in self.string
+
+    def __iter__(self):
+        """Yields the current char and moves the position onwards until eos."""
+        string = self.string
+        while not self.eos:
+            yield string[self.pos]
+            self.eat_length(1)
 
 class SourceLine(SourceString):
     """Contains an entire line of a source with handy line specific methods."""
