@@ -24,16 +24,16 @@ class Test_SourceString(unittest.TestCase):
         SRC = SourceString('hello\nworld')
 
         SRC.eat_string(SRC.get_length(5))
-        self.assertEqual(SRC.row, 0)
+        self.assertEqual(SRC.row, 1)
         self.assertEqual(SRC.col, 5)
 
         self.assertEqual(SRC.get_char(), '\n')
         SRC.eat_string(SRC.get_char())
-        self.assertEqual(SRC.row, 1)
+        self.assertEqual(SRC.row, 2)
         self.assertEqual(SRC.col, 0)
 
         SRC.eat_string(SRC.get_length(5))
-        self.assertEqual(SRC.row, 1)
+        self.assertEqual(SRC.row, 2)
         self.assertEqual(SRC.col, 5)
         self.assertEqual(SRC.get_char(), '')
 
@@ -41,7 +41,20 @@ class Test_SourceString(unittest.TestCase):
         SRC = SourceString('hello\nworld')
 
         SRC.eat_string('hello\nworld')
-        self.assertEqual(SRC.row, 1)
+        self.assertEqual(SRC.row, 2)
+        self.assertEqual(SRC.col, 5)
+        self.assertEqual(SRC.get_char(), '')
+
+    def test_eat_line(self):
+        SRC = SourceString('hello\nworld')
+
+        SRC.eat_line()
+        self.assertEqual(SRC.row, 2)
+        self.assertEqual(SRC.col, 0)
+        self.assertEqual(SRC.get_char(), 'w')
+
+        SRC.eat_line()
+        self.assertEqual(SRC.row, 2)
         self.assertEqual(SRC.col, 5)
         self.assertEqual(SRC.get_char(), '')
 
@@ -76,13 +89,6 @@ class Test_SourceString(unittest.TestCase):
         SRC.eat_length(5)
         self.assertTrue(SRC.eos)
 
-    def test_get_current_line(self):
-        SRC = SourceString('hello\nworld\nthis\nis\na\ntest')
-
-        self.assertEqual(repr(SRC.get_current_line()), 'hello\n')
-        SRC.eat_string('hello\n')
-        self.assertEqual(repr(SRC.get_current_line()), 'world\n')
-
     def test_get_line(self):
         SRC = SourceString('hello\nworld\nthis\nis\na\ntest')
 
@@ -90,47 +96,65 @@ class Test_SourceString(unittest.TestCase):
         self.assertEqual(repr(SRC.get_line(2)), 'this\n')
         self.assertEqual(SRC.get_line(20), None)
 
+    def test_get_current_line(self):
+        SRC = SourceString('hello\nworld\nthis\nis\na\ntest')
+
+        self.assertEqual(repr(SRC.get_current_line()), 'hello\n')
+        SRC.eat_string('hello\n')
+        self.assertEqual(repr(SRC.get_current_line()), 'world\n')
+
+    def test_get_lines(self):
+        SRC = SourceString('hello\nworld\nthis\nis\na\ntest')
+
+        lines = [str(x) for x in SRC.get_lines(1, 2)]
+        self.assertEqual(lines, ['1   |hello\n', '2   |world\n'])
+        lines = ''.join([repr(x) for x in SRC.get_lines(0, 2)])
+        self.assertEqual(lines, 'hello\nworld\n')
+
+        lines = [str(x) for x in SRC.get_lines(5, 6)]
+        self.assertEqual(lines, ['5   |a\n', '6   |test'])
+        lines = ''.join([repr(x) for x in SRC.get_lines(5, 6)])
+        self.assertEqual(lines, 'a\ntest')
+
+        self.assertEqual(SRC.get_lines(10, 20), None)
+
+    def test_get_all_lines(self):
+        base = 'hello\nworld\nthis\nis\na\ntest'
+        SRC = SourceString(base)
+
+        lines = [str(x) for x in SRC.get_all_lines()]
+        expected = ['1   |hello\n', '2   |world\n', '3   |this\n', '4   |is\n',
+            '5   |a\n', '6   |test']
+        self.assertEqual(lines, expected)
+        lines = ''.join([repr(x) for x in SRC.get_all_lines()])
+        self.assertEqual(lines, base)
+
     def test_get_surrounding_lines(self):
         SRC = SourceString('hello\nworld\nthis\nis\na\ntest')
 
         lines = [str(x) for x in SRC.get_surrounding_lines()]
-        self.assertEqual(lines, ['0   |hello\n', '1   |world\n'])
+        self.assertEqual(lines, ['1   |hello\n', '2   |world\n'])
         lines = ''.join([repr(x) for x in SRC.get_surrounding_lines()])
         self.assertEqual(lines, 'hello\nworld\n')
 
         SRC.eat_string('hello\nworld\n')
 
         lines = [str(x) for x in SRC.get_surrounding_lines()]
-        self.assertEqual(lines, ['1   |world\n', '2   |this\n', '3   |is\n'])
+        self.assertEqual(lines, ['2   |world\n', '3   |this\n', '4   |is\n'])
         lines = ''.join([repr(x) for x in SRC.get_surrounding_lines()])
         self.assertEqual(lines, 'world\nthis\nis\n')
 
         lines = [str(x) for x in SRC.get_surrounding_lines(1, 0)]
-        self.assertEqual(lines, ['1   |world\n', '2   |this\n'])
+        self.assertEqual(lines, ['2   |world\n', '3   |this\n'])
         lines = ''.join([repr(x) for x in SRC.get_surrounding_lines(1, 0)])
         self.assertEqual(lines, 'world\nthis\n')
 
         SRC.eat_string('this\nis\na\n')
         lines = [str(x) for x in SRC.get_surrounding_lines()]
-        self.assertEqual(lines, ['4   |a\n', '5   |test'])
+        self.assertEqual(lines, ['5   |a\n', '6   |test'])
         lines = ''.join([repr(x) for x in SRC.get_surrounding_lines()])
         self.assertEqual(lines, 'a\ntest')
 
-    def test_get_lines(self):
-        SRC = SourceString('hello\nworld\nthis\nis\na\ntest')
-
-        lines = [str(x) for x in SRC.get_lines(0, 1)]
-        self.assertEqual(lines, ['0   |hello\n', '1   |world\n'])
-        lines = ''.join([repr(x) for x in SRC.get_lines(0, 1)])
-        self.assertEqual(lines, 'hello\nworld\n')
-
-        lines = [str(x) for x in SRC.get_lines(4, 5)]
-        self.assertEqual(lines, ['4   |a\n', '5   |test'])
-        lines = ''.join([repr(x) for x in SRC.get_lines(4, 5)])
-        self.assertEqual(lines, 'a\ntest')
-
-        self.assertEqual(SRC.get_lines(10, 20), None)
-
     def test_match_string(self):
         MAT = SourceString('hello world\ntesting stuff')
 
@@ -186,6 +210,98 @@ class Test_SourceString(unittest.TestCase):
         self.assertEqual(MAT.count_indents_length(2, 1), (2, 3))
         self.assertEqual(MAT.count_indents_length(2), (1, 2))
 
+    def test_count_indents_last_line(self):
+        MAT = SourceString('Test100\n  test\n  more')
+
+        self.assertEqual(MAT.count_indents_last_line(2), 0)
+        MAT.eat_length(8)
+        self.assertEqual(MAT.count_indents_last_line(2), 0)
+        MAT.eat_length(7)
+        self.assertEqual(MAT.count_indents_last_line(2), 1)
+
+    def test_count_indents_length_last_line(self):
+        MAT = SourceString('Test100\n  test\n  more')
+
+        self.assertEqual(MAT.count_indents_length_last_line(2), (0, 0))
+        MAT.eat_length(8)
+        self.assertEqual(MAT.count_indents_length_last_line(2), (0, 0))
+        MAT.eat_length(7)
+        self.assertEqual(MAT.count_indents_length_last_line(2), (1, 2))
+
+    def test_skip_whitespace(self):
+        MAT = SourceString('  \tTest100')
+        MAT2 = SourceString('  \nTest100')
+
+        MAT.skip_whitespace()
+        self.assertEqual(MAT.get_char(), 'T')
+
+        MAT2.skip_whitespace()
+        self.assertEqual(MAT2.get_char(), '\n')
+        MAT2.skip_whitespace(1)
+        self.assertEqual(MAT2.get_char(), 'T')
+
+    def test_retrieve_tokens(self):
+        expected = [('identifier', 'key'), ('comma', ','),
+            ('identifier', 'value')]
+        MAT = SourceString('key, \n    value')
+        tokens = {'identifier': str.isalpha, 'comma': lambda x: x == ','}
+
+        geny = MAT.retrieve_tokens(MAT.match_function, tokens)
+        for i, token in enumerate(geny):
+            self.assertEqual(expected[i], token)
+
+        from partpy.spattern import identifier, qualified
+        MAT = SourceString('key, \n    value.test')
+        tokens = {'identifier': identifier, 'comma': ',', 'qualified': qualified}
+        expected = [('identifier', 'key'), ('comma', ','),
+            ('qualified', 'value.test')]
+
+        geny = MAT.retrieve_tokens(MAT.match_pattern, tokens)
+        for i, token in enumerate(geny):
+            self.assertEqual(expected[i], token)
+
+    def test_retrieve_tokens_newlines(self):
+        expected = [('identifier', 'key'), ('comma', ','), ('nl', '\n'),
+            ('identifier', 'value')]
+        MAT = SourceString('key, \n    value')
+        tokens = {'identifier': str.isalpha, 'comma': lambda x: x == ',',
+            'nl': lambda x: x == '\n'}
+
+        geny = MAT.retrieve_tokens(MAT.match_function, tokens, newlines = 0)
+        for i, token in enumerate(geny):
+            self.assertEqual(expected[i], token)
+
+        from partpy.spattern import alpha
+        MAT.reset_position()
+        tokens = {'identifier': alpha, 'comma': ',', 'nl': '\n'}
+
+        geny = MAT.retrieve_tokens(MAT.match_pattern, tokens, newlines = 0)
+        for i, token in enumerate(geny):
+            self.assertEqual(expected[i], token)
+
+    def test_retrieve_tokens_longest(self):
+        MAT = SourceString('test101')
+        expected = [('identifier', 'test101')]
+        from partpy.spattern import alpha, identifier, number
+        tokens = {'word': alpha, 'identifier': identifier, 'number': number}
+
+        geny = MAT.retrieve_tokens(MAT.match_pattern, tokens)
+        for i, token in enumerate(geny):
+            self.assertEqual(expected[i], token)
+
+        MAT.reset_position()
+        expected = [('word', 'test'), ('number', '101')]
+        geny = MAT.retrieve_tokens(MAT.match_pattern, tokens, longest = 0)
+        for i, token in enumerate(geny):
+            self.assertEqual(expected[i], token)
+
+    def test_iterator(self):
+        string = 'nekroze'
+        MAT = SourceString(string)
+
+        for i, char in enumerate(MAT):
+            self.assertEqual(string[i], char)
+
 
 if __name__ == "__main__":
     unittest.main()
